package higa.dbawt;

import java.awt.Checkbox;
import java.awt.CheckboxGroup;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import higa.sql.dataset.IDataSet;
import higa.sql.dataset.CheckAware;
import higa.sql.dataset.DataChangeEvent;
import higa.sql.dataset.DataChangeListener;
import higa.sql.dataset.NavigationEvent;
import higa.sql.dataset.NavigationListener;
import higa.sql.dataset.Status;

public class HCheckbox extends Checkbox
  implements CheckAware,
  DataChangeListener, NavigationListener,
  ItemListener {

  private IDataSet dataSet;
  private String columnName;
  private String checkValue;
  private String uncheckValue;

  private transient boolean addNotifyCalled = false;

  public HCheckbox() {
    addItemListener(this);
  }

  public synchronized void setCheckValue(
    String newCheckValue) {

    checkValue = newCheckValue;
    if (addNotifyCalled) {
      if (getState()) {
        setState(false);
      }

      openDataSet();
    }
  }

  public String getCheckValue() {
    return checkValue;
  }

  public synchronized void setUncheckValue(
    String newUncheckValue) {

    uncheckValue = newUncheckValue;
    if (addNotifyCalled) {
      openDataSet();
    }
  }

  public String getUncheckValue() {
    return uncheckValue;
  }

  public synchronized void setColumnName(
    String newColumnName) {

    if (getHCheckboxGroup() != null) {
      return;
    }

    columnName = newColumnName;
    if (addNotifyCalled) {
      openDataSet();
    }
  }

  public String getColumnName() {
    HCheckboxGroup hcg = getHCheckboxGroup();
    if (hcg != null) {
      return hcg.getColumnName();
    } else {
      return columnName;
    }
  }

  public synchronized void setDataSet(
    IDataSet newDataSet) {

    if (getHCheckboxGroup() != null) {
      return;
    }

    _setDataSet(newDataSet);
  }

  private synchronized void _setDataSet(
    IDataSet newDataSet) {

    if (dataSet != null) {
      dataSet.removeDataChangeListener(this);
      dataSet.removeNavigationListener(this);
    }

    dataSet = newDataSet;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }

    if (dataSet != null) {
      dataSet.addDataChangeListener(this);
      dataSet.addNavigationListener(this);
    }
  }

  public IDataSet getDataSet() {
    HCheckboxGroup hcg = getHCheckboxGroup();
    if (hcg != null) {
      return hcg.getDataSet();
    } else {
      return dataSet;
    }
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      openDataSet();
    }
  }

  public void dataChanged(DataChangeEvent e) {
    updateState();
  }

  public void navigated(NavigationEvent e) {
    updateState();
  }

  public void itemStateChanged(ItemEvent e) {
    CheckboxGroup cg = getCheckboxGroup();
    Checkbox current = null;
    if (cg != null) {
      current = cg.getSelectedCheckbox();
    }

    if (cg == null || this == current) {
      postState();
    }
  }

  public void setCheckboxGroup(CheckboxGroup cg) {
    HCheckboxGroup hcg = getHCheckboxGroup();
    if (hcg != null) {
      hcg.removeDataChangeListener(this);
      hcg.removeNavigationListener(this);
    }

    super.setCheckboxGroup(cg);
    hcg = getHCheckboxGroup();

    if (hcg != null) {
      _setDataSet(null);
      hcg.addDataChangeListener(this);
      hcg.addNavigationListener(this);

      if (addNotifyCalled) {
        openDataSet();
      }
    }
  }

  protected void updateState() {
    IDataSet ds = getDataSet();
    String cn = getColumnName();
    if (ds == null || cn == null) {
      return;
    }

    String value = ds.getString(cn);
    if ((value == null || value.length() == 0) &&
      (checkValue == null || checkValue.length() == 0) ||
      (value != null && value.equals(checkValue))) {
      setState(true);
    } else {
      setState(false);
    }
  }

  protected void postState() {
    IDataSet ds = getDataSet();
    String cn = getColumnName();
    if (ds == null || ds.getColumn(cn) == null) {
      return;
    }

    try {
      boolean value = getState();
      if (value) {
        ds.set(cn, checkValue);
      } else {
        ds.set(cn, uncheckValue);
      }
    } catch (Exception ex) {
      ex.printStackTrace();
      return;
    }
  }

  private HCheckboxGroup getHCheckboxGroup() {
    CheckboxGroup cg = super.getCheckboxGroup();
    if (cg != null &&
      cg instanceof HCheckboxGroup) {

      return (HCheckboxGroup)cg;
    } else {
      return null;
    }
  }

  private void openDataSet() {
    dataSet = getDataSet();
    if (dataSet == null || !addNotifyCalled) {
      return;
    }

    if (!dataSet.isOpen()) {
      try {
        dataSet.open();
      } catch (Exception ex) {
        ex.printStackTrace();
        return;
      }
    }

    if (dataSet.isOpen()) {
      updateState();
    }
  }
}