package higa.dbswing;

//import com.sun.java.swing.JToolBar;
//import com.sun.java.swing.Action;
//import com.sun.java.swing.AbstractAction;
//import com.sun.java.swing.ImageIcon;
import javax.swing.JToolBar;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;

import java.awt.GridLayout;
import java.awt.LayoutManager;

import java.awt.event.ActionEvent;

import java.net.URL;

import java.io.Serializable;

import higa.sql.dataset.DataSetAware;
import higa.sql.dataset.IDataSet;
import higa.sql.dataset.StatusEvent;
import higa.sql.dataset.StatusListener;

import higa.util.Resources;

public class HJNavigator
  extends JToolBar
  implements DataSetAware, StatusListener,
  Serializable {

  private static final Resources resources =
    new Resources(
    "higa.dbswing.resources.dbswing");

  public static final String FIRST =
    resources.getString("first");

  public static final String PRIOR =
    resources.getString("prior");

  public static final String NEXT =
    resources.getString("next");

  public static final String LAST =
    resources.getString("last");

  public static final String INSERT =
    resources.getString("insert");

  public static final String DELETE =
    resources.getString("delete");

  public static final String POST =
    resources.getString("post");

  public static final String CANCEL =
    resources.getString("cancel");

  public static final String COMMIT =
    resources.getString("commit");

  public static final String ROLLBACK =
    resources.getString("rollback");

  private final Action first = new FirstAction();
  private final Action prior = new PriorAction();
  private final Action next = new NextAction();
  private final Action last = new LastAction();
  private final Action insert = new InsertAction();
  private final Action delete = new DeleteAction();
  private final Action post = new PostAction();
  private final Action cancel = new CancelAction();
  private final Action commit = new CommitAction();
  private final Action rollback = new RollbackAction();

  private IDataSet dataSet;

  private transient boolean addNotifyCalled = false;

  public HJNavigator() {
    add(first);
    add(prior);
    add(next);
    add(last);
    add(insert);
    add(delete);
    add(post);
    add(cancel);
    add(commit);
    add(rollback);
  }

  public void setDataSet(IDataSet ds) {
    if (dataSet != null) {
      dataSet.removeStatusListener(this);
    }

    dataSet = ds;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }

    if (dataSet != null) {
      dataSet.addStatusListener(this);
    }
  }

  public IDataSet getDataSet() {
    return dataSet;
  }

  public void setRows(int rows) {
    LayoutManager layout = getLayout();
    if (layout instanceof GridLayout) {
      ((GridLayout)layout).setRows(rows);
    }
  }

  public int getRows() {
    LayoutManager layout = getLayout();
    if (layout instanceof GridLayout) {
      return ((GridLayout)layout).getRows();
    } else {
      return 0;
    }
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      setLayout(new GridLayout(2, 0));
      if (dataSet != null) {
        openDataSet();
      }
    }
  }

  public void statusMessage(StatusEvent e) {
    updateSelection();
  }

  protected void updateSelection() {
    first.setEnabled(false);
    prior.setEnabled(false);
    next.setEnabled(false);
    last.setEnabled(false);
    insert.setEnabled(true);
    delete.setEnabled(false);
    post.setEnabled(false);
    cancel.setEnabled(false);
    commit.setEnabled(false);
    rollback.setEnabled(false);

    if (!dataSet.isFirst()) {
      first.setEnabled(true);
      prior.setEnabled(true);
    }

    if (!dataSet.isLast()) {
      next.setEnabled(true);
      last.setEnabled(true);
    }

    if (dataSet.isEditing()) {
      post.setEnabled(true);
      cancel.setEnabled(true);
    }

    if (dataSet.isTransaction()) {
      commit.setEnabled(true);
      rollback.setEnabled(true);
    }

    if (!dataSet.isEmpty()) {
      delete.setEnabled(true);
    }
  }

  private void openDataSet() {
    if (dataSet == null || !addNotifyCalled) {
      return;
    }

    if (!dataSet.isOpen()) {
      try {
        dataSet.open();
      } catch (Exception ex) {
        ex.printStackTrace();
        return;
      }
    }

    if (dataSet.isOpen()) {
      updateSelection();
    }
  }

  abstract class NavigatorAction extends AbstractAction {

    public NavigatorAction(String name, String iconResource) {
      super(name);
      try {
        URL url = getClass().getResource(iconResource);
        super.putValue(SMALL_ICON, new ImageIcon(url));
      } catch (Exception ex) {}
    }
  }

  class FirstAction extends NavigatorAction {

    public FirstAction() {
      super(FIRST, "image/first.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.first();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  class PriorAction extends NavigatorAction {

    public PriorAction() {
      super(PRIOR, "image/prior.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.prior();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  class NextAction extends NavigatorAction {

    public NextAction() {
      super(NEXT, "image/next.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.next();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  class LastAction extends NavigatorAction {

    public LastAction() {
      super(LAST, "image/last.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.last();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  class InsertAction extends NavigatorAction {

    public InsertAction() {
      super(INSERT, "image/insert.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.insertRow();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  class DeleteAction extends NavigatorAction {

    public DeleteAction() {
      super(DELETE, "image/delete.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.deleteRow();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  class PostAction extends NavigatorAction {

    public PostAction() {
      super(POST, "image/post.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.post();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  class CancelAction extends NavigatorAction {

    public CancelAction() {
      super(CANCEL, "image/cancel.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.cancel();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  class CommitAction extends NavigatorAction {

    public CommitAction() {
      super(COMMIT, "image/commit.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.commit();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  class RollbackAction extends NavigatorAction {

    public RollbackAction() {
      super(ROLLBACK, "image/rollback.gif");
    }

    public void actionPerformed(ActionEvent e) {
      if (dataSet != null && dataSet.isOpen()) {
        try {
          dataSet.rollback();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    }
  }
}
