package higa.dbswing;

import java.io.Serializable;

import java.awt.Container;
import java.awt.Graphics;
import java.awt.SystemColor;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Dimension;
import java.awt.Rectangle;

import java.util.Vector;

//import com.sun.java.swing.JTable;
//import com.sun.java.swing.JScrollPane;
//import com.sun.java.swing.ListSelectionModel;
//import com.sun.java.swing.UIManager;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;

//import com.sun.java.swing.event.ListSelectionEvent;
//import com.sun.java.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

//import com.sun.java.swing.table.TableModel;
//import com.sun.java.swing.table.TableColumnModel;
//import com.sun.java.swing.table.TableColumn;
//import com.sun.java.swing.table.DefaultTableModel;
//import com.sun.java.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.DefaultTableColumnModel;

import higa.sql.dataset.DataSetAware;
import higa.sql.dataset.IDataSet;
import higa.sql.dataset.NavigationEvent;
import higa.sql.dataset.NavigationListener;

public class HJTable extends JTable
  implements DataSetAware, NavigationListener,
  ListSelectionListener, Serializable {

  private IDataSet dataSet;

  private transient boolean addNotifyCalled = false;
  private transient boolean navigating = false;

  public HJTable() {
  }

  public void updateUI() {
    super.updateUI();
    Container p = getParent();
    if (p instanceof JScrollPane) {
      JScrollPane sp = (JScrollPane)p;
      sp.setColumnHeaderView(getTableHeader());
      sp.getViewport().setBackingStoreEnabled(false);
      sp.setBorder(UIManager.getBorder("Table.scrollPaneBorder"));
    }
  }

  public void setDataSet(IDataSet ds) {
    if (dataSet != null) {
      dataSet.removeNavigationListener(this);
      getSelectionModel().removeListSelectionListener(this);
    }

    dataSet = ds;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }

    if (dataSet != null) {
      dataSet.addNavigationListener(this);
      getSelectionModel().addListSelectionListener(this);
    } else {
      setModel(new DefaultTableModel());
    }
  }

  public IDataSet getDataSet() {
    return dataSet;
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null) {
        openDataSet();
      }
    }

    updateUI();
  }

  private void openDataSet() {
    if (dataSet == null || !addNotifyCalled) {
      return;
    }

    if (!dataSet.isOpen()) {
      try {
        dataSet.open();
      } catch (Exception ex) {
        ex.printStackTrace();
        return;
      }
    }

    if (dataSet.isOpen()) {
      bindDataSet();
      repaint();
    }
  }

  private void bindDataSet() {
    if (dataSet != null) {
      HTableModel tm = new HTableModel(dataSet);
      setModel(tm);
      TableColumnModel tcm = new DefaultTableColumnModel();

      Graphics g = getGraphics();
      if (g != null) {
        Font f = getFont();
        FontMetrics fm = g.getFontMetrics(f);
        setRowHeight(fm.getHeight());
        int cCount = tm.getColumnCount();
        int rCount = Math.min(dataSet.getRowCount(), 10);
        for (int c = 0; c < cCount; c++) {
          String cn = tm.getColumnName(c);
          int size = fm.stringWidth(cn);
          for (int r = 0; r < rCount; r++) {
            Object value = tm.getValueAt(r, c);
            size = Math.max(size,
              fm.stringWidth(String.valueOf(value)));
          }

          TableColumn tc = new TableColumn(c, size + 10);
          tc.setHeaderValue(cn);
          tcm.addColumn(tc);
        }

        setColumnModel(tcm);
      }

      g.dispose();

      setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      setRowSelectionAllowed(false);
      setCellSelectionEnabled(true);
      setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
      setColumnSelectionInterval(0, 0);
      int rowIndex = dataSet.getRowIndex();
      setRowSelectionInterval(rowIndex, rowIndex);
      invalidate();
      Container p = getParent();
      if (p != null && p.isShowing()) {
        p.validate();
      }
    }
  }

  public void valueChanged(ListSelectionEvent e) {
    if (navigating) {
      navigating = false;
      return;
    }

    int rowIndex = getSelectedRow();
    if (dataSet == null) return;//TEST
    int dsRowIndex = dataSet.getRowIndex();
    if (rowIndex >= 0 && dsRowIndex != rowIndex) {
      try {
        dataSet.goToRow(rowIndex);
      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }

  public void navigated(NavigationEvent e) {
    navigating = true;
    int rowIndex = dataSet.getRowIndex();
    int columnIndex = getSelectedColumn();

    setRowSelectionInterval(rowIndex, rowIndex);
    Rectangle rect = getCellRect(
      rowIndex, columnIndex, true);

    scrollRectToVisible(rect);
    repaint();
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
    if (ps == null || ps.width < 5 || ps.height < 5) {
      ps = ps != null ? ps : new Dimension(100, 100);
      ps.width = ps.width > 100 ? ps.width : 100;
      ps.height = ps.height > 100 ? ps.height : 100;
    }

    return ps;
  }
}