package higa.dbswing;

//import com.sun.java.swing.JTextPane;
import javax.swing.JTextPane;

//import com.sun.java.swing.text.StyledDocument;
import javax.swing.text.StyledDocument;

import java.awt.Dimension;

import java.awt.event.KeyEvent;
import java.awt.event.FocusEvent;

import java.text.Format;

import java.io.Serializable;

import higa.sql.dataset.Column;
import higa.sql.dataset.DataChangeEvent;
import higa.sql.dataset.DataChangeListener;
import higa.sql.dataset.NavigationEvent;
import higa.sql.dataset.NavigationListener;
import higa.sql.dataset.IDataSet;
import higa.sql.dataset.ColumnAware;
import higa.sql.dataset.Status;

import higa.util.Formatter;

public class HJTextPane
  extends JTextPane
  implements ColumnAware,
  DataChangeListener, NavigationListener,
  Serializable {

  private IDataSet dataSet;
  private String columnName;
  private Format formatter;
  private String formatPattern;

  private String current;
  private transient boolean addNotifyCalled = false;

  public HJTextPane() {}

  public HJTextPane(StyledDocument doc) {
    super(doc);
  }

  public void setDataSet(IDataSet ds) {
    if (dataSet != null) {
      dataSet.removeDataChangeListener(this);
      dataSet.removeNavigationListener(this);
    }

    dataSet = ds;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }

    if (dataSet != null) {
      dataSet.addDataChangeListener(this);
      dataSet.addNavigationListener(this);
    }
  }

  public IDataSet getDataSet() {
    return dataSet;
  }

  public void setColumnName(String s) {
    columnName = s;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }
  }

  public String getColumnName() {
    return columnName;
  }

  public void setFormatter(Format formatter) {
    this.formatter = formatter;
    formatPattern = null;
    updateText();
  }

  public Format getFormatter() {
    return formatter;
  }

  public void setFormatPattern(String pattern) {
    formatPattern =
      pattern != null && pattern.length() != 0 ?
      pattern : null;

    formatter = null;
    updateText();
  }

  public String getFormatPattern() {
    return formatPattern;
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null) {
        openDataSet();
      }
    }
  }

  public void dataChanged(DataChangeEvent e) {
    updateText();
  }

  public void navigated(NavigationEvent e) {
    updateText();
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
    if (ps == null || ps.width < 10 || ps.height < 10) {
      ps = ps != null ? ps : new Dimension(100, 100);
      ps.width = ps.width > 100 ? ps.width : 100;
      ps.height = ps.height > 100 ? ps.height : 100;
    }
    
    return ps;
  }

  protected void updateText() {
    if (dataSet == null) {
      return;
    }

    Column column = dataSet.getColumn(columnName);
    if (column == null) {
      return;
    }

    if (formatter == null &&
      formatPattern != null) {

      int type = column.getType();
      formatter = Formatter.getDefaultFormatter(
        type, formatPattern);
    }

    String s = "";
    if (formatter == null) {
      s = dataSet.getString(columnName);
    } else {
      Object value = dataSet.get(columnName);
      s = Formatter.format(formatter, value);
    }

    setText(s);
    current = s;
  }

  protected boolean postText() {
    if (dataSet == null ||
      dataSet.getColumn(columnName) == null) {

      return false;
    }

    String s = getText();
    Object value = formatter == null ?
      s : Formatter.parseObject(formatter, s);

    try {
      dataSet.set(columnName, value);
    } catch (Exception ex) {
      ex.printStackTrace();
      return false;
    }

    return true;
  }

  protected void processKeyEvent(KeyEvent e) {
    super.processKeyEvent(e);
    if (dataSet == null || !dataSet.isOpen() ||
      dataSet.getColumn(columnName) == null || e.isConsumed()) {

      return;
    }

    switch (e.getID()) {
      case KeyEvent.KEY_PRESSED :
        try {
          boolean control = e.isControlDown();
          switch (e.getKeyCode()) {
            case KeyEvent.VK_ENTER :
              if (control) {
                postText();
              }

              break;
            case KeyEvent.VK_ESCAPE :
              updateText();
              break;
            case KeyEvent.VK_INSERT :
              if (control && postText()) {
                dataSet.insertRow();
              }

              break;
            case KeyEvent.VK_DELETE :
              if (control) {
                dataSet.deleteRow();
              }

              break;
            case KeyEvent.VK_PAGE_DOWN :
              if (control && postText()) {
                dataSet.next();
              }

              break;
            case KeyEvent.VK_PAGE_UP :
              if (control && postText()) {
                dataSet.prior();
              }

              break;
          }
        } catch (Exception ex) {
          ex.printStackTrace();
        }

        break;
    }
  }

  protected void processFocusEvent(FocusEvent e) {
    super.processFocusEvent(e);
    if (e.getID() == FocusEvent.FOCUS_LOST) {
      String text = super.getText();
      if (!text.equals(current)) {
        postText();
      }
    }
  }

  private void openDataSet() {
    if (dataSet == null || !addNotifyCalled) {
      return;
    }

    if (!dataSet.isOpen()) {
      try {
        dataSet.open();
      } catch (Exception ex) {
        ex.printStackTrace();
        return;
      }
    }

    if (dataSet.isOpen()) {
      updateText();
    }
  }
}
