package higa.jhcl.control;

import java.io.Serializable;

import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Dimension;

import higa.jhcl.view.HComponent;
import higa.jhcl.view.BorderItemPainter;

public class HPanel extends HComponent
  implements Serializable {

  public static final int FLAT    = 0;
  public static final int RAISED  = 1;
  public static final int LOWERED = 2;

  private int bevelInner = RAISED;
  private int bevelOuter = FLAT;
  private BorderItemPainter border;

  public HPanel() {
    setFocusAware(false);
    setupPainter();
  }

  public void setBevelInner(int bevelInner) {
    if (bevelInner >= FLAT && bevelInner <= LOWERED &&
      bevelInner != this.bevelInner) {

      this.bevelInner = bevelInner;
      setupPainter();
    }
  }

  public int getBevelInner() {
    return bevelInner;
  }

  public void setBevelOuter(int bevelOuter) {
    if (bevelOuter >= FLAT && bevelOuter <= LOWERED &&
      bevelOuter != this.bevelOuter) {

      this.bevelOuter = bevelOuter;
      setupPainter();
    }
  }

  public int getBevelOuter() {
    return bevelOuter;
  }

  public Insets getInsets() {
    Insets i = super.getInsets();
    Insets b = border != null ? border.getInsets() :
      new Insets(0,0,0,0);

    return new Insets(
      i.top + b.top, i.left + b.left,
      i.bottom + b.bottom, i.right + b.right);
  }

  public Dimension getPreferredSize() {
    Dimension d = super.getPreferredSize();
    if (d.width <= 10) {
      d.width = 100;
    }

    if (d.height <= 10) {
      d.height = 100;
    }

    return d;
  }

  public void paint(Graphics g) {
    super.paint(g);
    if (border != null) {
      Dimension size = getSize();
      Rectangle rect = new Rectangle(
        0, 0, size.width, size.height);

      border.paint(null, g, rect, 0, this);
    }
  }

  protected void setupPainter() {
    if (bevelInner == FLAT && bevelOuter == FLAT) {
      border = null;
    } else {
      int style = 0;
      switch (bevelInner) {
        case RAISED:
          style |= BorderItemPainter.INNER_RAISED;
          break;
        case LOWERED:
          style |= BorderItemPainter.INNER_SUNKEN;
          break;
      }

      switch (bevelOuter) {
        case RAISED:
          style |= BorderItemPainter.OUTER_RAISED;
          break;
        case LOWERED:
          style |= BorderItemPainter.OUTER_SUNKEN;
          break;
      }

      border = new BorderItemPainter(style);
    }

    repaint();
  }
}