package higa.jhcl.model;

import java.io.Serializable;

import java.util.Vector;

import higa.util.EventMulticaster;

public class BasicVectorContainer
  implements VectorModel, Serializable, Cloneable {

  private Vector array = new Vector();
  private EventMulticaster modelCaster =
    new EventMulticaster();

  private boolean eventEnabled = true;

  public BasicVectorContainer() {}

  public BasicVectorContainer(BasicVectorContainer bvc) {
    array = (Vector)bvc.array.clone();
  }

  public Object clone() {
    return new BasicVectorContainer(this);
  }

  public Object[] getItems() {
    Object[] items = new Object[array.size()];
    array.copyInto(items);
    return items;
  }

  public void setItems(Object[] newItems) {
    int count = newItems.length;
    for (int i = 0; i < count; i++) {
      array.addElement(newItems[i]);
    }

    processModelEvent(new VectorModelEvent(
      this, VectorModelEvent.STRUCTURE_CHANGED));
  }

  public Object get(int index) {
    if (canSet(index)) {
      return array.elementAt(index);
    } else {
      return null;
    }
  }

  public int getCount() {
    return array.size();
  }

  public int find(Object data) {
    return array.indexOf(data);
  }

  public void setCount(int count) {
    array.setSize(count);
  }

  public void addModelListener(VectorModelListener listener) {
    modelCaster.add(listener);
  }

  public void removeModelListener(VectorModelListener listener) {
    modelCaster.remove(listener);
  }

  public boolean canSet(int index) {
    return index >= 0 && index < array.size();
  }

  public void set(int index, Object object) {
    if (canSet(index)) {
      array.setElementAt(object, index);
    } else {
      array.setSize(index + 1);
      array.setElementAt(object, index);
    }

    processModelEvent(new VectorModelEvent(
      this, VectorModelEvent.ITEM_CHANGED, index));
  }

  public void addItem(Object object) {
    array.addElement(object);
    processModelEvent(new VectorModelEvent(
      this, VectorModelEvent.ITEM_ADDED, array.size()));
  }

  public void addItem(int index, Object object) {
    array.insertElementAt(object, index);
    processModelEvent(new VectorModelEvent(
      this, VectorModelEvent.ITEM_ADDED, index));
  }

  public void remove(int index) {
    array.removeElementAt(index);
    processModelEvent(new VectorModelEvent(
      this, VectorModelEvent.ITEM_REMOVED, index));
  }

  public void removeAll() {
    if (array.size() > 0) {
      array.removeAllElements();
      processModelEvent(new VectorModelEvent(
        this, VectorModelEvent.STRUCTURE_CHANGED));
    }
  }

  public void setEventEnabled(boolean enabled) {
    eventEnabled = enabled;
  }

  protected void processModelEvent(VectorModelEvent e) {
    if (eventEnabled && modelCaster.hasListeners()) {
      modelCaster.dispatch(e);
    }
  }
}