package higa.jhcl.model;

import java.io.Serializable;

import higa.util.EventMulticaster;

public class SingleMatrixSelection
  implements MatrixSelection, Serializable {

  private MatrixLocation cell;
  private EventMulticaster selectionCaster =
    new EventMulticaster();

  private boolean eventEnabled = true;

  public SingleMatrixSelection() {}

  public SingleMatrixSelection(MatrixLocation location) {
    cell = location;
  }

  public boolean contains(MatrixLocation location) {
    return location.equals(cell);
  }

  public boolean contains(int rowIndex, int columnIndex) {
    if (cell != null) {
      return cell.rowIndex == rowIndex &&
        cell.columnIndex == columnIndex;
    } else {
      return false;
    }
  }

  public int getCount() {
    return cell != null ? 1 : 0;
  }

  public MatrixLocation[] getAll() {
    MatrixLocation[] contents =
      new MatrixLocation[getCount()];

    if (cell != null) {
      contents[0] = cell;
    }

    return contents;
  }

  public void addSelectionListener(
    MatrixSelectionListener listener) {

    selectionCaster.add(listener);
  }

  public void removeSelectionListener(
    MatrixSelectionListener listener) {

    selectionCaster.remove(listener);
  }

  public void set(MatrixLocation[] locations) {
    add(locations);
  }

  public void add(MatrixLocation location) {
    if (!location.equals(cell)) {
      cell = new MatrixLocation(location);
      processSelectionEvent(
        new MatrixSelectionEvent(
        this, SelectionEvent.SELECTION_CHANGED));
    }
  }

  public void add(int rowIndex, int columnIndex) {
    if (cell != null) {
      if (!contains(rowIndex, columnIndex)) {
        cell.rowIndex = rowIndex;
        cell.columnIndex = columnIndex;
        processSelectionEvent(
          new MatrixSelectionEvent(
          this, SelectionEvent.SELECTION_CHANGED));
      }
    } else {
      cell = new MatrixLocation(rowIndex, columnIndex);
      processSelectionEvent(
        new MatrixSelectionEvent(
        this, SelectionEvent.SELECTION_CHANGED));
    }
  }

  public void add(MatrixLocation[] locations) {
    if (locations.length > 0) {
      if (!locations[0].equals(cell)) {
        cell = new MatrixLocation(locations[0]);
        processSelectionEvent(
          new MatrixSelectionEvent(
          this, SelectionEvent.SELECTION_CHANGED));
      }
    }
  }

  public void addRange(MatrixLocation begin,
    MatrixLocation end) {

    if (!end.equals(cell)) {
      cell = new MatrixLocation(end);
      processSelectionEvent(
        new MatrixSelectionEvent(
        this, SelectionEvent.SELECTION_CHANGED));
    }
  }

  public void addRange(int beginRowIndex, int beginColumnIndex,
    int endRowIndex, int endColumnIndex) {

    if (cell != null) {
      if (!contains(endRowIndex, endColumnIndex)) {
        cell.rowIndex    = endRowIndex;
        cell.columnIndex = endColumnIndex;
        processSelectionEvent(
          new MatrixSelectionEvent(
          this, SelectionEvent.SELECTION_CHANGED));
      }
    } else {
      cell = new MatrixLocation(endRowIndex, endColumnIndex);
      processSelectionEvent(
        new MatrixSelectionEvent(
        this, SelectionEvent.SELECTION_CHANGED));
    }
  }

  public void remove(MatrixLocation location) {
    if (location.equals(cell)) {
      cell = null;
      processSelectionEvent(
        new MatrixSelectionEvent(
        this, SelectionEvent.SELECTION_CHANGED));
    }
  }

  public void remove(int rowIndex, int columnIndex) {
    if (cell != null && contains(rowIndex, columnIndex)) {
      cell = null;
      processSelectionEvent(
        new MatrixSelectionEvent(
        this, SelectionEvent.SELECTION_CHANGED));
    }
  }

  public void remove(MatrixLocation[] locations) {
    if (cell != null) {
      for (int i = 0 ; i < locations.length ; i++ ) {
        if (locations[i].equals(cell)) {
          cell = null;
          processSelectionEvent(
            new MatrixSelectionEvent(
            this, SelectionEvent.SELECTION_CHANGED));
            
          break;
        }
      }
    }
  }

  public void removeRange(MatrixLocation begin,
    MatrixLocation end) {

    removeRange(begin.rowIndex, begin.columnIndex,
      end.rowIndex, end.columnIndex);
  }

  public void removeRange(int beginRowIndex, int beginColumnIndex,
    int endRowIndex, int endColumnIndex) {
    if (cell != null) {
      if (cell.rowIndex >= beginRowIndex &&
        cell.rowIndex <= endRowIndex &&
        cell.columnIndex >= beginColumnIndex &&
        cell.columnIndex <= endColumnIndex) {

        cell = null;
        processSelectionEvent(
          new MatrixSelectionEvent(
          this, SelectionEvent.SELECTION_CHANGED));
      }
    }
  }

  public void removeAll() {
    if (cell != null) {
      cell = null;
      processSelectionEvent(
        new MatrixSelectionEvent(
        this, SelectionEvent.SELECTION_CLEARED));
    }
  }
  
  public void setEventEnabled(boolean enabled) {
    eventEnabled = enabled;
  }

  protected void processSelectionEvent(
    MatrixSelectionEvent e) {

    if (eventEnabled && selectionCaster.hasListeners()) {
      selectionCaster.dispatch(e);
    }
  }
}