package higa.jhcl.view;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.SystemColor;

import higa.jhcl.model.ItemPainter;
import higa.jhcl.model.ItemPaintSite;

public class ButtonItemPainter
  implements ItemPainter, Serializable {

  static final int margin = 2 + 1;

  private ItemPainter painter;

  public ButtonItemPainter() {}

  public ButtonItemPainter(ItemPainter painter) {
    this.painter = painter;
  }

  public void setPainter(ItemPainter painter) {
    this.painter = painter;
  }

  public ItemPainter getPainter() {
    return painter;
  }

  public Dimension getPreferredSize(
    Object data, Graphics g,
    int state, ItemPaintSite site) {

    if (painter != null) {
      Dimension d =
        painter.getPreferredSize(
        data, g, state, site);

      return new Dimension(
        d.width + margin * 2,
        d.height + margin * 2);
    } else {
      return new Dimension(
        margin * 2, margin * 2);
    }
  }

  public void paint(Object data, Graphics g,
    Rectangle rect, int state,
    ItemPaintSite site) {

    g.setColor(SystemColor.control);
    //g.fillRect(rect.x, rect.y, rect.width, rect.height);
    Rectangle borderRect = new Rectangle(rect);
    borderRect.width--;
    borderRect.height--;

    if ((state & ItemPainter.SELECTED) == 0) {
      g.draw3DRect(borderRect.x, borderRect.y,
        borderRect.width, borderRect.height, true);
    } else {
      g.draw3DRect(borderRect.x, borderRect.y,
        borderRect.width, borderRect.height, false);
    }

    if (painter != null) {
      Rectangle faceRect =
        new Rectangle(rect);

      if ((state & SELECTED) == 0) {
        faceRect.grow(-margin, -margin);
      } else {
        faceRect = new Rectangle(
          rect.x + margin + 1, rect.y + margin + 1,
          rect.width - margin * 2 - 1,
          rect.height - margin * 2 - 1);
      }

      painter.paint(data, g, faceRect, state, site);
    }
  }

  private void writeObject(ObjectOutputStream s)
    throws IOException {

    s.defaultWriteObject();
    s.writeObject(
      painter instanceof Serializable ? painter : null);
  }

  private void readObject(ObjectInputStream s)
    throws IOException, ClassNotFoundException {
    
    s.defaultReadObject();
    Object data = s.readObject();
    if (data instanceof ItemPainter) {
      painter = (ItemPainter)data;
    }
  }
}