package higa.jhcl.view;

import java.awt.Insets;

import java.io.Serializable;

import higa.jhcl.model.BasicVectorContainer;
import higa.jhcl.model.ItemPainter;
import higa.jhcl.model.ItemEditor;

import higa.util.Alignment;

public class ColumnViewManager
  extends BasicVectorContainer
  implements SizeVector, Serializable {

  private int defaultColumnWidth = 100;
  private int defaultAlignment =
    Alignment.LEFT | Alignment.MIDDLE;

  public ColumnViewManager() {}

  public void setColumnViews(ColumnView[] cvs) {
    if (cvs == null) {
      cvs = new ColumnView[0];
    }

    super.removeAll();
    int count = cvs.length;
    for (int i = 0; i < count; i++) {
      set(i, cvs[i]);
    }
  }

  public ColumnView[] getColumnViews() {
    int count = getCount();
    ColumnView[] cvs = new ColumnView[count];
    for (int i = 0; i < count; i++) {
      cvs[i] = getColumnView(i);
    }

    return cvs;
  }


  public void setColumnView(int index, ColumnView cv) {
    set(index, cv);
  }

  public ColumnView getColumnView(int index) {
    ColumnView cv = (ColumnView)super.get(index);
    if (cv == null) {
      return getDefaultColumnView(index);
    }

    return new ColumnView(cv);
  }

  public void addColumnView(int index) {
    ColumnView cv = getDefaultColumnView(index);
    addItem(cv);
  }

  public void dropColumnView(int index) {
    remove(index);
  }

  public void setSize(int index, int size) {
    ColumnView cv = getColumnView(index);
    if (cv == null) {
      cv = getDefaultColumnView(index);
    }

    cv.setWidth(size);
    set(index, cv);
  }

  public int getSize(int index) {
    ColumnView cv = getColumnView(index);
    if (cv == null) {
      return defaultColumnWidth;
    }

    int width = cv.getWidth();
    width = width <= 0 ? defaultColumnWidth : width;
    return width;
  }

  public int getSizeUpTo(int index) {
    int size = 0;
    int last = Math.min(index, super.getCount());
    for (int i = 0; i < last; i++) {
      size += getSize(i);
    }

    return size;
  }

  public int getAlignment(int index) {
    ColumnView cv = getColumnView(index);
    if (cv == null) {
      return defaultAlignment;
    } else {
      return cv.getAlignment();
    }
  }

  public ItemPainter getPainter(int index) {
    ColumnView cv = getColumnView(index);
    return cv != null ? cv.getPainter() : null;
  }

  public ItemEditor getEditor(int index) {
    ColumnView cv = getColumnView(index);
    return cv != null ? cv.getEditor() : null;
  }

  public Object get(int index) {
    ColumnView cv = getColumnView(index);
    return cv.getCaption();
  }

  public void setCount(int newCount) {
    int oldCount = getCount();
    super.setCount(newCount);
    if (oldCount < newCount) {
      for (int i = oldCount; i < newCount; i++) {
        ColumnView cv = getDefaultColumnView(i);
        super.set(i , cv);
      }
    }
  }

  public ColumnView getDefaultColumnView(int index) {
    ColumnView cv = new ColumnView();
    String name = "Column " + String.valueOf(index);

    cv.setName(name);
    cv.setAlignment(defaultAlignment);
    cv.setItemMargins(new Insets(0,2,0,2));
    cv.setWidth(defaultColumnWidth);

    return cv;
  }
}